unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    GroupBox3: TGroupBox;
    MinButton: TButton;
    MidButton: TButton;
    MaxButton: TButton;
    GroupBox1: TGroupBox;
    LoLabel: TLabel;
    LoTrack: TTrackBar;
    GroupBox2: TGroupBox;
    HiLabel: TLabel;
    HiTrack: TTrackBar;
    BlurbMemo: TMemo;
    Panel1: TPanel;
    SpacerBevel: TBevel;
    WriteOneButton: TButton;
    WriteAllButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure LoTrackChange(Sender: TObject);
    procedure HiTrackChange(Sender: TObject);
    procedure MaxButtonClick(Sender: TObject);
    procedure MidButtonClick(Sender: TObject);
    procedure MinButtonClick(Sender: TObject);
    procedure WriteOneButtonClick(Sender: TObject);
    procedure WriteAllButtonClick(Sender: TObject);
  protected
    procedure BuildCalFile;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit, WaitUnit;

const
  NybbleToHex: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

const
  //Currently in quarter-seconds.
  EstEraseTime = 2;
  Est1To1Time  = 17;
  EstWriteTime = 54;

var
  DeviceIndex: LongWord = diOnly;
  Channels: Integer;
  CalFN: String;

procedure DumpStringToFile(S: AnsiString; Path: String);
var
  Fil: TFileStream;

begin
  Fil := TFileStream.Create(Path, fmCreate or fmShareDenyWrite);
  try
    Fil.Write(S[1], Length(S));
  finally
    Fil.Free;
  end;
end;

function DumpFileToString(Path: String): AnsiString;
var
  Fil: TFileStream;
  L: Integer;

begin
  Fil := TFileStream.Create(Path, fmOpenRead or fmShareDenyNone);
  try
    L := Fil.Size;
    SetString(Result, nil, L);
    Fil.Read(Result[1], L);
  finally
    Fil.Free;
  end;
end;

function GetTempDir: String;
var
  L: Integer;

begin
  SetString(Result, nil, MAX_PATH);
  L := GetTempPath(Length(Result), @Result[1]);
  SetLength(Result, L);
  Result := IncludeTrailingBackslash(Result);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status, PID: LongWord;
  DetectForm: TDetectForm;
  CalData: Word;
  CalDataSize: LongWord;

begin
  WriteAllButton.Align := alBottom;
  SpacerBevel.Align := alBottom;
  WriteOneButton.Align := alClient;

  CalFN := GetTempDir + 'Cal.txt';

  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or (PID < $8070) or (PID > $807F) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $8070, $8071, $8078, $8079: Channels := 16;
    $8072, $8073, $807A, $807B: Channels := 12;
    $8074, $8075, $807C, $807D: Channels := 8;
    $8076, $8077, $807E, $807F: Channels := 4;
    else Channels := 0;
  end;

  CalData := $07FE;
  CalDataSize := SizeOf(CalData);
  AIOUSB_FlashRead(DeviceIndex, $0000, CalDataSize, @CalData);
  LoTrack.Position := CalData xor $FFFF;
  LoTrackChange(LoTrack);

  CalData := $F801;
  CalDataSize := SizeOf(CalData);
  AIOUSB_FlashRead(DeviceIndex, $1FFFE, CalDataSize, @CalData);
  HiTrack.Position := CalData xor $FFFF;
  HiTrackChange(HiTrack);

  DACSetBoardRange(DeviceIndex, 0);

  BlurbMemo.Lines.Text := 'The calibration values from the device have been loaded in the "Low" and "High" sliders to the left.'
            + #13#10#13#10'Use the "Ch 0" buttons to set channel 0 to its minimum, middle, or maximum value, and measure the error. Convert it to counts; 1 count is .076mV at 0-5V, .153mV at 0-10V or 5V, or .305mV at 10V.'
            + #13#10#13#10'Use the "Low" slider to adjust the low end of the calibration, and similarly the "High" slider for the high end.'
            + #13#10#13#10'Click "Write To Channel 0", and wait(about 15 seconds).'
            + #13#10#13#10'Try the "Ch 0" buttons again. Repeat until channel 0 is calibrated.'
            + #13#10#13#10'Click "Write To All Channels" and wait(3-4 minutes for 16 channels).'
  ;
end;

procedure TMainForm.LoTrackChange(Sender: TObject);
begin
  LoLabel.Caption := IntToHex(LoTrack.Position xor $FFFF, 4);
end;

procedure TMainForm.HiTrackChange(Sender: TObject);
begin
  HiLabel.Caption := IntToHex(HiTrack.Position xor $FFFF, 4);
end;

procedure TMainForm.MaxButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, 0, $FFFF);
end;

procedure TMainForm.MidButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, 0, $8000);
end;

procedure TMainForm.MinButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, 0, $0000);
end;

procedure TMainForm.WriteOneButtonClick(Sender: TObject);
begin
  BuildCalFile;

  WaitForm.GoWait(EstWriteTime);
  DACSetChannelCal(DeviceIndex, 0, PChar(CalFN));
  Sleep(1000);
  WaitForm.Hide;
end;

procedure TMainForm.WriteAllButtonClick(Sender: TObject);
var
  iChannel: Integer;

begin
  BuildCalFile;

  for iChannel := 0 to Channels-1 do begin
    WaitForm.GoWait((Channels - iChannel) * EstWriteTime);
    DACSetChannelCal(DeviceIndex, 0, PChar(CalFN));
    Sleep(1000);
  end;
  WaitForm.Hide;
end;

procedure TMainForm.BuildCalFile;
var
  I, J: Integer;
  LoCt, HiCt: Integer;
  F: Double;
  Buf: AnsiString;

begin
  LoCt := LoTrack.Position xor $FFFF;
  HiCt := HiTrack.Position xor $FFFF;

  SetLength(Buf, $70000);

  for I := 0 to $FFFF do begin
    F := (I - 0) / $FFFF;
    F := F * (HiCt - LoCt) + LoCt;
    J := Round(F);
    //Fil.WriteStr('$' + IntToHex(J, 4) + #13#10);
    Buf[I * 7 + 7] := #10;
    Buf[I * 7 + 6] := #13;
    Buf[I * 7 + 5] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 4] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 3] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 2] := NybbleToHex[J and $F];
    Buf[I * 7 + 1] := '$';
  end;

  DumpStringToFile(Buf, CalFN);
end;

end.
